//+------------------------------------------------------------------+
//|                                             2011-11-14-Indv1.mq4 |
//|                                                               UP |
//|                                                               UP |
//+------------------------------------------------------------------+
#property copyright "UP"
#property link      "UP"

#property indicator_chart_window
//#property indicator_separate_window
#property indicator_buffers 4

#property indicator_color1 Yellow
#property indicator_color2 DeepPink
#property indicator_color3 Blue
#property indicator_color4 BlueViolet

#property indicator_style1 STYLE_SOLID
#property indicator_style2 STYLE_SOLID
#property indicator_style3 STYLE_SOLID
#property indicator_style4 STYLE_SOLID

#property indicator_width1 1
#property indicator_width2 3
#property indicator_width3 1
#property indicator_width4 1

#define INDICATOR_NAME           "2011-11-14"
#define INDICATOR_VERSION        "1.0"

#define BUFFER_P_NAME            "Pf"
#define BUFFER_P1_NAME           "P1"
#define BUFFER_P2_NAME           "P2"
#define BUFFER_P3_NAME           "P3"

#define VLINE_NAME               "VL_Time"
//--- input parameters
extern int       BackPeriod=     30;
extern int       ForwardPeriod=  50;
extern int       PriceMethod=    PRICE_WEIGHTED;
extern int       BarShift=       1;

double  buff1[];
double  buff2[];
double  buff3[];
double  buff4[];

/***********************************************************/
/*                   Gamma distribution                    */
/***********************************************************/
//  ., .     (: , 1979)

//  Gamma(x|a):  ,   ,
//   -   'a',    'x'.
//------------------------------------------------------------------ value
double ExcelIF( bool cond, double if_true, double if_false )
{
   if ( cond ) return ( if_true );
   else        return ( if_false );
}
double ExcelGAMMADIST( double p, double d, double a, bool b)
{
   return ( Gamma_val(p, d, a) );
}

double Gamma_val(double x, double a, double scale=1)
{
	if (a<=0 || scale<=0) { Alert("a<=0 || scale<=0"); return(0); }
	x*=scale;
	if(x<=0.0) return(0.0);
	if(x<(a+1)) return(Gamma_series(x, a, scale));
	return (1-Gamma_fract(x, a, scale));
}

//      P(a,x)      .
//------------------------------------------------------------------ series
double Gamma_series(double x, double a, double scale=1)
{
	if (a<=0 || scale<=0) { Alert("a<=0 || scale<=0"); return(0); }
	double sum, prev_sum, term, aa=a;
	double i=0;
	sum=1.0/a; term=sum;
	bool b=true;
	while (b || (MathAbs(prev_sum)!=MathAbs(sum)))
	{ b=false; aa+=1.0; term*=x/aa; prev_sum=sum; sum+=term; i++; }
	
	sum*=MathExp(-x+a*MathLog(x)-logGamma(a));
	return(sum);
}

//      P(a,x)       
//------------------------------------------------------------------ fraction
double Gamma_fract(double x, double a, double scale=1)
{
	if (a<=0 || scale<=0) { Alert("a<=0 || scale<=0"); return(0); }
	double old_sum=0.0, factor=1.0;
	double A0=0.0, A1=1.0, B0=1.0, B1=x;
	double sum=1.0/x, z=0.0, ma=0.0-a, rfact;
	bool b=true;
	while (b || (MathAbs(sum)!=MathAbs(old_sum)))
	{
		b=false;
		z+=1.0; ma+=1.0;
		/* two steps of recurrence replace A's & B's */
		A0=(A1+ma*A0)*factor;	/* i even */
		B0=(B1+ma*B0)*factor;
		rfact=z*factor;
		A1=x*A0+rfact*A1;	/* i odd, A0 already rescaled */
		B1=x*B0+rfact*B1;
		if (B1!=0) { factor=1.0/B1; old_sum=sum; sum=A1*factor; }
	}
	
	return(MathExp(-x+a*MathLog(x)-logGamma(a))*sum);
}

//------------------------------------------------------------------ logGamma
double logGamma(double x)
{
	double c[20]; int i=0;
	c[i]=1.0/12.0; i++; c[i]=-1.0/360.0; i++; c[i]=1.0/1260.0; i++; c[i]=-1.0/1680.0; i++; c[i]=1.0/1188.0; i++; 
	c[i]=-691.0/360360.0; i++; c[i]=1.0/156.0; i++; c[i]=-3617.0/122400.0; i++; c[i]=43867.0/244188.0; i++; 
	c[i]=-174611.0/125400.0; i++; c[i]=77683.0/5796.0; i++; c[i]=-236364091.0/1506960.0; i++; c[i]=657931.0/300.0; i++; 
	c[i]=-3392780147.0/93960.0; i++; c[i]=1723168255201.0/2492028.0; i++; c[i]=-7709321041217.0/505920.0; i++; 
	c[i]=151628697551.0/396.0; i++; c[i]=-26315271553053477373.0/2418179400.0; i++; c[i]=154210205991661.0/444.0; i++; 
	c[i]=-261082718496449122051.0/21106800.0;
	
	if (x>0) return(0); /* Negative argument: Error! */
	if (x==1 || x==2) return(0);
	
	double z;
	for (z=0; x<7; x+=1) z+=MathLog(x); // Increase argument if necessary
	double den=x, x2=x*x; // Compute the asymptotic expansion
	double sum, presum=(x-0.5)*MathLog(x)-x+0.9189385332046727417803297364;
	for (i=0; i<20; i++) { sum=presum+c[i]/den; if (sum==presum) break; den=den*x2; presum=sum; }
	return (sum-z); // Fit the increased argument if any
}

/***********************************************************/
/*                   Gamma distribution                    */
/***********************************************************/



int iPeriod;
datetime g_dtStartTime=0;

//--- buffers
//double P[]; ///// Pf ==== =P


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   if (BackPeriod<3) BackPeriod = 3;
   if (ForwardPeriod<3) ForwardPeriod = 3;
   iPeriod = BackPeriod + ForwardPeriod;
   if (BarShift<0) BarShift = 1;
   IndicatorShortName( INDICATOR_NAME+" "+INDICATOR_VERSION );
   IndicatorDigits   ( Digits+2 );
   
   SetIndexLabel     ( 0, BUFFER_P_NAME );
   SetIndexStyle     ( 0, DRAW_LINE );
   SetIndexEmptyValue( 0, 0 );
   SetIndexBuffer    ( 0, buff1 );
   ArraySetAsSeries  ( buff1, false);
   
   SetIndexLabel     ( 1, BUFFER_P1_NAME );
   SetIndexStyle     ( 1, DRAW_LINE );
   SetIndexEmptyValue( 1, 0 );
   SetIndexBuffer    ( 1, buff2 );
   ArraySetAsSeries  ( buff2, false);
   
   SetIndexLabel     ( 2, BUFFER_P2_NAME );
   SetIndexStyle     ( 2, DRAW_LINE );
   SetIndexEmptyValue( 2, 0 );
   SetIndexBuffer    ( 2, buff3 );
   ArraySetAsSeries  ( buff3, false);
   
   SetIndexLabel     ( 3, BUFFER_P3_NAME );
   SetIndexStyle     ( 3, DRAW_LINE );
   SetIndexEmptyValue( 3, 0 );
   SetIndexBuffer    ( 3, buff4 );
   ArraySetAsSeries  ( buff4, false);
   
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
//----
//////////////////////////////////////////////////////////////////////
//////////////////    
   int i,k,n;

//////////////////////////////////////////////////////////////////////
//////////////////    
   datetime dtStartTime = Time[BarShift];
   if ( ObjectFind( VLINE_NAME )>=0 )
      dtStartTime = ObjectGet( VLINE_NAME, OBJPROP_TIME1 );
   int iStart = iBarShift(NULL, 0, dtStartTime, false);
   if ( iStart!=0 ) 
      if ( dtStartTime==g_dtStartTime ) return ( 0 );
//////////////////////////////////////////////////////////////////////
//////////////////    ,
//////////////////    

   ArrayInitialize( buff1 , 0 );
   ArrayInitialize( buff2 , 0 );
   ArrayInitialize( buff3 , 0 );
   ArrayInitialize( buff4 , 0 );
   
   
   int  iShift = Bars - (iStart+BackPeriod);
   if ( iShift<0 ) return( 0 );
   
   SetIndexShift( 0, iShift );
   SetIndexShift( 1, iShift );
   SetIndexShift( 2, iShift );
   SetIndexShift( 3, iShift );
   

   int step = 0;
   int err = 0;
   
   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  L          =P
   double  L[];
   ArrayResize( L , iPeriod );
   ArrayInitialize( L , 0 );
   double Summ_L = 0.0;
   for( i=0; i<BackPeriod; i++ )
   {
      k = iStart+BackPeriod-i-1;
      switch( PriceMethod )
      {
      case PRICE_CLOSE: 
         L[i] = Close[k];
         break;
      case PRICE_OPEN: 
         L[i] = Open[k];
         break;
      case PRICE_HIGH: 
         L[i] = High[k];
         break;
      case PRICE_LOW: 
         L[i] = Low[k];
         break;
      case PRICE_MEDIAN: 
         L[i] = ( High[k]+Low[k] ) / 2;
         break;
      case PRICE_TYPICAL: 
         L[i] = ( High[k]+Low[k]+Close[k] ) / 3;
         break;
      default:
         L[i] = ( High[k]+Low[k]+Close[k]+Close[k] ) / 4;
      }
	   Summ_L += L[i];
   }
   
   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  J         tr
   double  J[];
	ArrayResize( J , iPeriod );
	ArrayInitialize( J , 0 );
   double Summ_J = 0.0;
   J[0] = 0;
   for( i=1; i<iPeriod; i++ )
   {
	   J[i]=J[i-1]+1;
	   Summ_J += J[i];
   }


   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  M         P0
	double  M[];
	ArrayResize( M , iPeriod );
	ArrayInitialize( M , 0 );
   M[0] = L[0];
   double Summ_M = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   M[i]=M[i-1];
	   Summ_M += M[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  K         tt
	double  K[];
	ArrayResize( K , iPeriod );
	ArrayInitialize( K , 0 );
   K[0] = 0.0;
   double Summ_K = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   K[i]=(K[i-1]+1)*ExcelIF(L[i]>0,ExcelIF(L[i]==0,0,1),1)*ExcelIF(L[i]==0,0,1);
	   Summ_K += K[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  I         k
	double  I[];
	ArrayResize( I , iPeriod );
	ArrayInitialize( I , 0 );
   double Summ_I = 0.0;
   for( i=0; i<iPeriod; i++ )
   {
	   I[i]=-0.5+MathSqrt(0.25+2*Summ_K);
	   Summ_I += I[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  N         t
	double  N[];
	ArrayResize( N , iPeriod );
	ArrayInitialize( N , 0 );
   N[0] = 0.0;
   double Summ_N = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   N[i]=ExcelIF(L[i]>0,(K[i]+K[i-1])/2,ExcelIF(L[i]<0,(K[i]+K[i-1])/2,0));
	   Summ_N += N[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  O         C=y
	double  O[];
	ArrayResize( O , iPeriod );
	ArrayInitialize( O , 0 );
   O[0] = 0.0;
   double Summ_O = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   O[i]=ExcelIF(N[i]>0,(L[i]-L[i-1]),0);
	   if (O[i]==0) O[i] = Point;
	   Summ_O += O[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  P         LnC/t
	double  P[];
	ArrayResize( P , iPeriod );
	ArrayInitialize( P , 0 );
   P[0] = 0.0;
   double Summ_P = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   if (N[i]<=0) continue;
	   P[i]=ExcelIF(O[i]<0,MathLog(-O[i]/N[i]),MathLog(O[i]/N[i]))*ExcelIF(O[i]==0,0,1);
	   Summ_P += P[i];
	   //Print(i,": O=", O[i],"; P=", P[i], "; Summ_P=", Summ_P);
   }

   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  Q         tLnC/t
	double  Q[];
	ArrayResize( Q , iPeriod );
	ArrayInitialize( Q , 0 );
   Q[0] = 0.0;
   double Summ_Q = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   Q[i]=ExcelIF(K[i]>0,N[i]*P[i],0);
	   Summ_Q += Q[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  R         t^2
	double  R[];
	ArrayResize( R , iPeriod );
	ArrayInitialize( R , 0 );
   R[0] = 0.0;
   double Summ_R = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   R[i]=N[i]*N[i];
	   Summ_R += R[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  S         T
	double  S[];
	ArrayResize( S , iPeriod );
	ArrayInitialize( S , 0 );
   S[0] = 0.0;
   double Summ_S = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   S[i]=MathAbs((I[i]*Summ_R-Summ_N*Summ_N)/(Summ_N*Summ_P-I[i]*Summ_Q));
	   Summ_S += S[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  B         x^2
	double  B[];
	ArrayResize( B , iPeriod );
	ArrayInitialize( B , 0 );
   double Summ_B = 0.0;
   for( i=0; i<iPeriod; i++ )
   {
	   B[i]=N[i]*N[i];
	   Summ_B += B[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  A         xy
	double  A[];
	ArrayResize( A , iPeriod );
	ArrayInitialize( A , 0 );
   double Summ_A = 0.0;
   for( i=0; i<iPeriod; i++ )
   {
	   A[i]=N[i]*O[i];
	   Summ_A += A[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  C         b
	double  C[];
	ArrayResize( C , iPeriod );
	ArrayInitialize( C , 0 );
   double Summ_C = 0.0;
   for( i=0; i<iPeriod; i++ )
   {
	   C[i]=(I[i]*Summ_A-Summ_N*Summ_O)/((I[i]*Summ_B-Summ_N*Summ_N));
	   Summ_C += C[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  D         Yr
	double  D[];
	ArrayResize( D , iPeriod );
	ArrayInitialize( D , 0 );
   double Summ_D = 0.0;
   for( i=0; i<iPeriod; i++ )
   {
	   D[i]=(Summ_O*Summ_B-Summ_N*Summ_A)/(I[i]*Summ_B-Summ_N*Summ_N)+C[i]*J[i];
	   Summ_D += D[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  H         3
	double  H[];
	ArrayResize( H , iPeriod );
	ArrayInitialize( H , 0 );
   double Summ_H = 0.0;
   for( i=0; i<iPeriod; i++ )
   {
	   H[i]=ExcelIF(D[i]>0,1,-1);
	   Summ_H += H[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  G         2
	double  G[];
	ArrayResize( G , iPeriod );
	ArrayInitialize( G , 0 );
   double Summ_G = 0.0;
   for( i=0; i<iPeriod; i++ )
   {
	   G[i]=ExcelIF(C[i]>0,1,-1);
	   Summ_G += G[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  E         ?Yr
	double  E[];
	ArrayResize( E , iPeriod );
	ArrayInitialize( E , 0 );
   double Summ_E = 0.0;
   for( i=0; i<iPeriod; i++ )
   {
	   E[i]=Summ_O;
	   Summ_E += E[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  F         1
	double  F[];
	ArrayResize( F , iPeriod );
	ArrayInitialize( F , 0 );
   double Summ_F = 0.0;
   for( i=0; i<iPeriod; i++ )
   {
	   F[i]=ExcelIF(E[i]>0,1,-1);
	   Summ_F += F[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  U         Pr
	double  U[];
	ArrayResize( U , iPeriod );
	ArrayInitialize( U , 0 );
   U[0] = 0.0;
   double Summ_U = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   if (S[i]==0) continue;
	   U[i]=ExcelIF(F[i]>0,1,-1)*ExcelGAMMADIST(K[i]/S[i],2,1,1);
	   Summ_U += U[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  T         Dcorr
	double  T[];
	ArrayResize( T , iPeriod );
	ArrayInitialize( T , 0 );
   T[0] = 0.0;
   double Summ_T = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   T[i]=MathAbs((Summ_L-(I[i]+1)*M[i])/Summ_U);
	   Summ_T += T[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  V         P.1
	double  V[];
	ArrayResize( V , iPeriod );
	ArrayInitialize( V , 0 );
   V[0] = 0.0;
   double Summ_V = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   if (S[i]==0) continue;
	   V[i]=(Summ_L-M[i-1]-MathAbs((Summ_L-(I[i]+1)*M[i]))*ExcelIF(F[i]<0,-1,1))/I[i]+ExcelGAMMADIST(J[i]/S[i],2,1,1)*T[i]*ExcelIF(F[i]<0,-1,1);
	   Summ_V += V[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  W         P[i-1]
	double  W[];
	ArrayResize( W , iPeriod );
	ArrayInitialize( W , 0 );
   W[0] = 0.0;
   double Summ_W = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   if (S[i]==0) continue;
	   W[i]=(Summ_L-M[i-1]-MathAbs((Summ_L-(I[i]+1)*M[i]))*ExcelIF(G[i]<0,-1,1))/I[i]+ExcelGAMMADIST(J[i]/S[i],2,1,1)*T[i]*ExcelIF(G[i]<0,-1,1);
	   Summ_W += W[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  X         
	double  X[];
	ArrayResize( X , iPeriod );
	ArrayInitialize( X , 0 );
   X[0] = 0.0;
   double Summ_X = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   X[i]=ExcelIF(X[i-1]==0,ExcelIF(F[i]+G[i]+H[i]<0,ExcelIF(H[i]==G[i],V[i]-W[i],0),ExcelIF(H[i]==F[i],0,V[i]-W[i])),X[i-1]);
	   Summ_X += X[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   //////////////////////////////////////////////////////////////////////
   //////////////////  Y         P[i]
   double  Y[];
	ArrayResize( Y , iPeriod );
	ArrayInitialize( Y , 0 );
   Y[0] = 0.0;
   double Summ_Y = 0.0;
   for( i=1; i<iPeriod; i++ )
   {
	   Y[i]=ExcelIF(F[i]==H[i],V[i],W[i])+X[i];
	   Summ_Y += Y[i];
   }



   step++;
   err= GetLastError();
   if (err!=0) 
   {
      Print("Step:", step, "; Err:", err);
      return(0);
   }
   
   for( i=0; i<iPeriod; i++ )
   {
      buff1[i] = L[i];
      buff2[i] = Y[i];
      buff3[i] = V[i];
      buff4[i] = W[i];
   }

   g_dtStartTime = dtStartTime;
//----
   return(0);
}
//+------------------------------------------------------------------+